"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.renamingRule = void 0;
const schematics_1 = require("@angular-devkit/schematics");
const ng_morph_1 = require("ng-morph");
const format_files_1 = require("./format-files");
const dependencies_1 = require("@schematics/angular/utility/dependencies");
const get_latest_node_version_1 = require("./get-latest-node-version");
function renamingRule(packageNames, renames, classRenames) {
    const getRename = configureRenames(renames, classRenames);
    return () => {
        return (0, schematics_1.chain)([
            (tree) => {
                (0, ng_morph_1.setActiveProject)((0, ng_morph_1.createProject)(tree, '/', ['**/*.ts']));
                for (let packageName of packageNames) {
                    const imports = (0, ng_morph_1.getImports)('**/*.ts', {
                        moduleSpecifier: packageName,
                    });
                    const newImports = new Map();
                    for (const importDeclaration of imports) {
                        const namedImports = importDeclaration.getNamedImports();
                        for (const namedImport of namedImports) {
                            const oldName = namedImport.getName();
                            const rename = getRename(oldName);
                            if (rename == null) {
                                continue;
                            }
                            const filePath = importDeclaration
                                .getSourceFile()
                                .getFilePath()
                                .toString();
                            const key = `${filePath}__${rename.moduleSpecifier}`;
                            const namedImportConfig = {
                                name: rename.namedImport,
                            };
                            if (namedImport.getAliasNode()) {
                                // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
                                namedImportConfig.alias = namedImport.getAliasNode().getText();
                            }
                            if (newImports.has(key)) {
                                // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
                                const value = newImports.get(key);
                                newImports.set(key, [...value, namedImportConfig]);
                            }
                            else {
                                newImports.set(key, [namedImportConfig]);
                            }
                            renameReferences(namedImport, oldName, rename.namedImport);
                            namedImport.remove();
                        }
                        if (importDeclaration.getNamedImports().length === 0) {
                            importDeclaration.remove();
                        }
                    }
                    for (const [key, namedImports] of newImports.entries()) {
                        const [filePath, moduleSpecifier] = key.split('__');
                        (0, ng_morph_1.addImports)(filePath, {
                            namedImports: namedImports,
                            moduleSpecifier: moduleSpecifier,
                        });
                    }
                }
                (0, ng_morph_1.saveActiveProject)();
            },
            replacePackageJsonDepsRule(),
            (0, format_files_1.formatFiles)(),
            (_, context) => {
                context.logger.info(`⚠️ Please run "npm install" to install the new dependencies!`);
                context.logger.info(`🎉️ Migration completed successfully!`);
            }
        ]);
    };
}
exports.renamingRule = renamingRule;
function renameReferences(importSpecifier, oldName, newName) {
    importSpecifier
        .getNameNode()
        .findReferencesAsNodes()
        .forEach((ref) => {
        if (ref.getText() === oldName) {
            ref.replaceWithText(newName);
        }
    });
}
/**
 * Returns a function that can be used to rename imports.
 *
 * @param renames
 * @param classRenames
 */
function configureRenames(renames, classRenames) {
    return (namedImport) => {
        if (renames[namedImport] == null) {
            return null;
        }
        const newNamedImport = Array.isArray(renames[namedImport])
            ? getNewClassName(renames[namedImport][0], classRenames)
            : getNewClassName(namedImport, classRenames);
        return {
            namedImport: newNamedImport,
            moduleSpecifier: Array.isArray(renames[namedImport])
                ? renames[namedImport][1]
                : renames[namedImport],
        };
    };
}
function getNewClassName(oldName, classRenames) {
    return classRenames[oldName] || oldName;
}
function replacePackageJsonDepsRule() {
    return (tree, context) => __awaiter(this, void 0, void 0, function* () {
        const oldDepName = 'ngx-isr';
        const newDepName = '@rx-angular/isr';
        (0, dependencies_1.removePackageJsonDependency)(tree, oldDepName);
        context.logger.info(`❌️ Removed dependency ${oldDepName}`);
        const version = yield (0, get_latest_node_version_1.getLatestNodeVersion)(newDepName);
        (0, dependencies_1.addPackageJsonDependency)(tree, {
            name: newDepName,
            type: dependencies_1.NodeDependencyType.Default,
            overwrite: true,
            version,
        });
        context.logger.info(`✅️ Added dependency ${newDepName}@${version}`);
    });
}
